/*
 * leader_engine.h
 */

#include "seal.h"

#define LEADER_JOIN_EVENT 1
#define LEADER_LEAVE_EVENT 2

typedef struct leader_control_s * LEADER_control_p_t;
typedef struct leader_control_s {
	int leader_flag;			// 1 if the member itself is the leader, 
								// 0 if some other member is the leader, and I
								// know who he is
								// -1 if some other member is the leader, but
								// I don't know who he is

	char leader_guid[SEAL_GUID_LEN];	// leader's guid (if leader_flag == 1, 
										// leader_guid should be guid_g)
	
	int leader_seq;				// the sequence number of sending out rekey 
								// message (required for leader only)

	pthread_mutex_t lock;		// lock
} LEADER_control_t;

LEADER_control_p_t LEADER_init();

/* leader management */
int LEADER_elect(LEADER_control_p_t leader, char const* my_guid, int no_curr,
		char* const* curr_guids, int const* curr_ts);
void LEADER_set_leader(LEADER_control_p_t leader, char const* my_guid,
		char const* leader_guid);

/* sequence number management */
void LEADER_seq_max(LEADER_control_p_t leader, int recv_seq);
void LEADER_seq_increment(LEADER_control_p_t leader);
int LEADER_seq_retrieve(LEADER_control_p_t leader);

/* checking functions */
int LEADER_is_changed(LEADER_control_p_t leader);
int LEADER_is_elect(LEADER_control_p_t leader, char const* my_guid,
		char const* mem_guid, int event);
int LEADER_is_leader(LEADER_control_p_t leader);
int LEADER_is_valid(LEADER_control_p_t leader, char const* claimed_leader);

/* destroy functions */
void LEADER_destroy();

