/*
 * member_engine.h
 */

#include "seal.h"

/***** constants *****/
#define MEMBER_CURR_LIST		0x01
#define MEMBER_JOIN_LIST		0x02
#define MEMBER_LEAVE_LIST		0x03

/* forward declaration of MEMLIST */
typedef struct memlist_entry_s * MEMLIST_entry_p_t;
typedef struct memlist_entry_s {
	char guid[SEAL_GUID_LEN];	// guid of the member
	int ts;						// timestamp of insertion
	MEMLIST_entry_p_t prev;
	MEMLIST_entry_p_t next;
} MEMLIST_entry_t;

typedef struct memlist_control_s * MEMLIST_control_p_t;
typedef struct memlist_control_s {
	MEMLIST_entry_p_t head;	// the first entry in the member list
	MEMLIST_entry_p_t tail;	// the last entry in the member list
	int size;
} MEMLIST_control_t;

typedef struct member_control_s * MEMBER_control_p_t;
typedef struct member_control_s {
	MEMLIST_control_p_t curr_memlist;
	MEMLIST_control_p_t join_memlist;
	MEMLIST_control_p_t leave_memlist;
	pthread_mutex_t lock;
} MEMBER_control_t;

/***** function prototypes ******/
MEMBER_control_p_t MEMBER_init();
void MEMBER_insert(MEMBER_control_p_t member_engine, int list_type, 
		char const* guid, int ts);
int MEMBER_delete(MEMBER_control_p_t member_engine, int list_type, 
		char const* guid);
void MEMBER_delete_all(MEMBER_control_p_t member_engine, int list_type);

int MEMBER_get_size(MEMBER_control_p_t member_engine, int list_type);
int MEMBER_get_all(MEMBER_control_p_t member_engine, int list_type, 
		char** guids, int* ts);

int MEMBER_is_empty(MEMBER_control_p_t member_engine, int list_type);

void MEMBER_print(MEMBER_control_p_t member_engine, int list_type);

void MEMBER_destroy(MEMBER_control_p_t member_engine);

