/*
 * rekey_queue.c
 *
 * - holding the rekey packets that are not yet sent
 */

#include "seal.h"

/*
 * init the rekey queue
 */
RQUEUE_control_p_t RQUEUE_init() {
	RQUEUE_control_p_t rekey_queue;

	rekey_queue = (RQUEUE_control_p_t)malloc(sizeof(RQUEUE_control_t));

	rekey_queue->head = NULL;
	rekey_queue->tail = NULL;
	rekey_queue->size = 0;
	rekey_queue->stop_flag = 0;	
	
	pthread_mutex_init(&(rekey_queue->lock), NULL);
	pthread_cond_init(&(rekey_queue->empty_cond), NULL);
	pthread_cond_init(&(rekey_queue->stop_cond), NULL);

	return rekey_queue;
}

/*-------------------------------------------------------------------------
 *						QUEUE MANAGEMENT
 *-------------------------------------------------------------------------*/
void RQUEUE_enq(RQUEUE_control_p_t rekey_queue, char rekey_mode,
		char const* qsubtree_join, char const* qsubtree_leave) {

	RQUEUE_node_p_t new_node_p;

	// malloc the new node
	new_node_p = (RQUEUE_node_p_t)malloc(sizeof(RQUEUE_node_t));
	new_node_p->rekey_mode = rekey_mode;
	if (qsubtree_join != NULL) {
		memcpy(new_node_p->qsubtree_join, qsubtree_join, SEAL_GUID_LEN);
		memset(new_node_p->qsubtree_leave, 0, SEAL_GUID_LEN);
	} else if (qsubtree_leave != NULL) {
		memcpy(new_node_p->qsubtree_leave, qsubtree_leave, SEAL_GUID_LEN);
		memset(new_node_p->qsubtree_join, 0, SEAL_GUID_LEN);
	}
	new_node_p->next = NULL;

	// add to the queue
	if (rekey_queue->head == NULL) {
		rekey_queue->head = new_node_p;
	} else {
		rekey_queue->tail->next = new_node_p;
	}
	rekey_queue->tail = new_node_p;
	rekey_queue->size++;
}


/*
 * Dequeue a rekey message from the queue
 */
void RQUEUE_deq(RQUEUE_control_p_t rekey_queue, char* rekey_mode,
		char* qsubtree_join, char* qsubtree_leave) {

	RQUEUE_node_p_t del_node_p;

	if (rekey_queue->size <= 0) {
		debug_err("Dequeue from an empty rekey queue\n");
		return;
	}

	// get the head information
	del_node_p = rekey_queue->head;
	*rekey_mode = del_node_p->rekey_mode;
	memcpy(qsubtree_join, del_node_p->qsubtree_join, SEAL_GUID_LEN);
	memcpy(qsubtree_leave, del_node_p->qsubtree_leave, SEAL_GUID_LEN);

	// remove the head	
	if (rekey_queue->head == rekey_queue->tail) {
		rekey_queue->head = NULL;
		rekey_queue->tail = NULL;
		rekey_queue->size = 0;
	} else {
		rekey_queue->head = del_node_p->next;
		rekey_queue->size--;
	}

	free(del_node_p);
}


/*
 * Set the stop flag
 */
void RQUEUE_set_stopflag(RQUEUE_control_p_t rekey_queue, int f) {
	rekey_queue->stop_flag = f;
}

/*
 * Return the size of the current queue
 */
int RQUEUE_get_size(RQUEUE_control_p_t rekey_queue) {
	return rekey_queue->size;
}

/*
 * Return the flag of the queue
 */
int RQUEUE_get_stopflag(RQUEUE_control_p_t rekey_queue) {
	return rekey_queue->stop_flag;
}

/*-------------------------------------------------------------------------
 *						DESTROY
 *-------------------------------------------------------------------------*/
/*
 * Destroy the rekey queue
 */
void RQUEUE_destroy(RQUEUE_control_p_t rekey_queue) {
	RQUEUE_node_p_t del_node_p;

	while (rekey_queue->head != NULL) {
		del_node_p = rekey_queue->head;
		rekey_queue->head = del_node_p->next;
		free(del_node_p);
	}
	free(rekey_queue);
}


