/*
 * rekey_queue.h
 */

/*** structure declaration ***/
typedef struct rekey_bcast_queue_node_s * RQUEUE_node_p_t;
typedef struct rekey_bcast_queue_node_s {
	char rekey_mode;
	char qsubtree_join[SEAL_GUID_LEN];		// used by qbatch algorithm only
	char qsubtree_leave[SEAL_GUID_LEN];		// used by qbatch algorithm only	
	RQUEUE_node_p_t next;
} RQUEUE_node_t;

typedef struct rekey_bcast_queue_s * RQUEUE_control_p_t;
typedef struct rekey_bcast_queue_s {
	RQUEUE_node_p_t head;
	RQUEUE_node_p_t tail;
	int size;
	int stop_flag;				// determine if it should stop sending
	pthread_mutex_t lock;
	pthread_cond_t empty_cond;	// wait if the condition is empty
	pthread_cond_t stop_cond;	// wait if the condition is stop sending
} RQUEUE_control_t;

/*** function prototypes ***/
RQUEUE_control_p_t RQUEUE_init();

/* queue management */
void RQUEUE_enq(RQUEUE_control_p_t rekey_queue, char rekey_mode,
		char const* qsubtree_join, char const* qsubtree_leave);
void RQUEUE_deq(RQUEUE_control_p_t rekey_queue, char* rekey_mode,
		char* qsubtree_join, char* qsubtree_leave);
void RQUEUE_set_stopflag(RQUEUE_control_p_t rekey_queue, int f);
int RQUEUE_get_size(RQUEUE_control_p_t rekey_queue);
int RQUEUE_get_stopflag(RQUEUE_control_p_t rekey_queue);

/* destroy */
void RQUEUE_destroy(RQUEUE_control_p_t rekey_queue);

